import { Address, CustomerAddress } from '@bigcommerce/checkout-sdk';
import React, { FunctionComponent, memo, useState } from 'react';
import { useSelector } from 'react-redux'

import { preventDefault } from '@bigcommerce/checkout/dom-utils';
import { TranslatedString } from '@bigcommerce/checkout/locale';
import { PoweredByPayPalFastlaneLabel, usePayPalFastlaneAddress } from '@bigcommerce/checkout/paypal-fastlane-integration';

import B3Checkout, { AddressFilterInput } from '../b3checkout';
import { DropdownTrigger } from '../ui/dropdown';

import AddressSelectButton from './AddressSelectButton';
import AddressType from './AddressType';
import isEqualAddress from './isEqualAddress';
import StaticAddress from './StaticAddress';

import './AddressSelect.scss';

export interface AddressSelectProps {
    addresses: CustomerAddress[];
    selectedAddress?: Address;
    type: AddressType;
    onSelectAddress(address: Address): void;
    onUseNewAddress(currentAddress?: Address): void;
}

interface B3Props {
    b3Company: any;
    isB3User: boolean;
}

const AddressSelectMenu: FunctionComponent<AddressSelectProps & B3Props> = ({
    addresses,
    onSelectAddress,
    onUseNewAddress,
    selectedAddress,
    isB3User,
    type,
}) => {
    const [disPlayAddress, setDisPlayAddress] = useState(addresses)

    return (
        <ul
            className="dropdown-menu instrumentSelect-dropdownMenu"
            id="addressDropdown"
        >
            { isB3User ? 
                <AddressFilterInput 
                    onChange={(q: string) => {
                        setDisPlayAddress(B3Checkout.utils.util.addressesFilterHandler(q, addresses))
                    }}
                /> : 
                (
                    <li className="dropdown-menu-item dropdown-menu-item--select">
                        <a
                            data-test="add-new-address"
                            href="#"
                            onClick={preventDefault(() => onUseNewAddress(selectedAddress))}
                        >
                            <TranslatedString id="address.enter_address_action" />
                        </a>
                    </li>
                ) 
            }
            { disPlayAddress.map(address => (
                <li className="dropdown-menu-item dropdown-menu-item--select" key={address.id}>
                    <a href="#" onClick={preventDefault(() => onSelectAddress(address))}>
                        <StaticAddress address={address} type={type} />
                    </a>
                </li>
            )) }
        </ul>
    )
};

const AddressSelect = ({
    addresses,
    selectedAddress,
    type,
    onSelectAddress,
    onUseNewAddress,
}: AddressSelectProps) => {
    const { shouldShowPayPalFastlaneLabel } = usePayPalFastlaneAddress();

    const {
        isB3User,
        company: b3Company,
    } = useSelector(({
        global: {
            isB3User,
        },
        company,
    }: any) => ({
        isB3User,
        company,
    }))

    const b2bProps = {
        isB3User,
        b3Company,
    }

    const handleSelectAddress = (newAddress: Address) => {
        if (!isEqualAddress(selectedAddress, newAddress)) {
            onSelectAddress(newAddress);
        }
    };

    const handleUseNewAddress = () => {
        onUseNewAddress(selectedAddress);
    };

    return (
        <div className="form-field">
            <div className="dropdown--select">
                <DropdownTrigger
                    dropdown={
                        <AddressSelectMenu
                            {...b2bProps}
                            addresses={addresses}
                            onSelectAddress={handleSelectAddress}
                            onUseNewAddress={handleUseNewAddress}
                            selectedAddress={selectedAddress}
                            type={type}
                        />
                    }
                >
                    <AddressSelectButton
                        {...b2bProps}
                        addresses={addresses}
                        selectedAddress={selectedAddress}
                        type={type}
                    />
                </DropdownTrigger>
            </div>

            {shouldShowPayPalFastlaneLabel && <PoweredByPayPalFastlaneLabel />}
        </div>
    );
}

export default memo(AddressSelect);
